@@ -3,10 +3,15 @@ import pytest
 from emmet.core.charge_density import ChgcarDataDoc
 from mp_api.routes.charge_density import ChargeDensityRester
 
-import inspect
 import typing
 
-resters = [ChargeDensityRester()]
+
+@pytest.fixture
+def rester():
+    rester = ChargeDensityRester()
+    yield rester
+    rester.session.close()
+
 
 excluded_params = [
     "sort_fields",
@@ -14,6 +19,7 @@ excluded_params = [
     "num_chunks",
     "all_fields",
     "fields",
+    "return",
 ]
 
 sub_doc_fields = []  # type: list
@@ -26,13 +32,8 @@ custom_field_tests = {}  # type: dict
 @pytest.mark.skipif(
     os.environ.get("MP_API_KEY", None) is None, reason="No API key found."
 )
-@pytest.mark.parametrize("rester", resters)
 def test_client(rester):
-    # Get specific search method
-    search_method = None
-    for entry in inspect.getmembers(rester, predicate=inspect.ismethod):
-        if "search" in entry[0] and entry[0] != "search":
-            search_method = entry[1]
+    search_method = rester.search
 
     if search_method is not None:
         # Get list of parameters
@@ -84,8 +85,7 @@ def test_client(rester):
                 )
 
 
-def test_download_for_task_ids(tmpdir):
-    rester = resters[0]
+def test_download_for_task_ids(tmpdir, rester):
 
     n = rester.download_for_task_ids(
         task_ids=["mp-655585", "mp-1057373", "mp-1059589", "mp-1440634", "mp-1791788"],
@@ -96,8 +96,7 @@ def test_download_for_task_ids(tmpdir):
     assert "mp-1791788.json.gz" in files
 
 
-def test_extract_s3_url_info():
-    rester = resters[0]
+def test_extract_s3_url_info(rester):
 
     url_doc_dict = {
         "task_id": "mp-1896591",
